/**
 * \file grouping/ptm/ptmsamplescan.cpp
 * \date 23/7/2017
 * \author Olivier Langella
 * \brief collection of PTM peptides sharing the same spectrum (sample scan)
 */

/*******************************************************************************
* Copyright (c) 2017 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#include "ptmsamplescan.h"
#include "../../grouping/ptm/ptmgroupingexperiment.h"


QColor PtmSampleScan::_color_best = QColor("red");
QColor PtmSampleScan::_color_other_best = QColor("#ff7878");
QColor PtmSampleScan::_color_no_best = QColor("yellow");


PtmSampleScan::PtmSampleScan(const PeptideMatch * p_peptide_match) {
    _peptide_match_list.push_back(p_peptide_match);
}

PtmSampleScan::PtmSampleScan(const PtmSampleScan& other) {
}

PtmSampleScan::~PtmSampleScan() {
}
const PeptideMatch * PtmSampleScan::getRepresentativePeptideMatch() const {
    return _peptide_match_list[0];
}
bool PtmSampleScan::add(const PeptideMatch * p_peptide_match) {
    if(getRepresentativePeptideMatch()->getHashSampleScan() != p_peptide_match->getHashSampleScan()) {
        return false;
    }
    if(getRepresentativePeptideMatch()->getPeptideXtpSp().get()->getSequence() != p_peptide_match->getPeptideXtpSp().get()->getSequence()) {
        return false;
    }
    if(getRepresentativePeptideMatch()->getPeptideXtpSp().get()->getMass() != p_peptide_match->getPeptideXtpSp().get()->getMass()) {
        return false;
    }
    _peptide_match_list.push_back(p_peptide_match);

    //sort list
    std::sort(_peptide_match_list.begin(),_peptide_match_list.end(),[](const PeptideMatch * first, const PeptideMatch * second) {
        return (first->getEvalue() < second->getEvalue()) ;
    });

    return true;
}

std::vector<unsigned int> PtmSampleScan::getBestPtmPositionList(const PtmGroupingExperiment * p_ptm_grouping_experiment) const {
    return p_ptm_grouping_experiment->getPtmPositions(getRepresentativePeptideMatch());
}

std::vector<unsigned int> PtmSampleScan::getObservedPtmPositionList(const PtmGroupingExperiment * p_ptm_grouping_experiment) const {
    std::vector<unsigned int> position_list;

    for (const PeptideMatch * p_peptide_match: _peptide_match_list) {
        std::vector<unsigned int> positionb_list = p_ptm_grouping_experiment->getPtmPositions(p_peptide_match);
        for (unsigned int position: positionb_list) {
            position_list.push_back(position);
        }
    }
    std::sort(position_list.begin(), position_list.end());
    auto last = std::unique(position_list.begin(), position_list.end());
    position_list.erase(last, position_list.end());
    return position_list;
}

const QString PtmSampleScan::getHtmlSequence(const PtmGroupingExperiment * p_ptm_grouping_experiment) const
{
    const PeptideMatch * p_representative_peptide = getRepresentativePeptideMatch();
    size_t pep_size = p_representative_peptide->getPeptideXtpSp().get()->size();
    //qDebug() << "ProteinMatch::getCoverage begin prot_size=" << prot_size << " " << _protein_sp.get()-//>getSequence();
    if (pep_size == 0) return 0;
    bool best_bool[pep_size] = {false};
    bool other_best_bool[pep_size] = {false};
    bool nobest_bool[pep_size] = {false};
    double best_evalue = p_representative_peptide->getEvalue();


    for (auto & p_peptide_match : _peptide_match_list) {
        bool is_best= false;
        if (p_peptide_match->getEvalue() == best_evalue) {
            is_best= true;
        }
        std::vector<unsigned int> position_list = p_ptm_grouping_experiment->getPtmPositions(p_peptide_match);
        for (unsigned int position :position_list) {
            if (p_representative_peptide == p_peptide_match) {
                best_bool[position] = true;
            }
            else if (is_best) {
                other_best_bool[position] = true;
            }
            else {
                nobest_bool[position] = true;
            }
        }
    }
    QString sequence = p_representative_peptide->getPeptideXtpSp().get()->getSequence();
    QString sequence_html;
    for (unsigned int i=0; i < pep_size; i++) {
        if (best_bool[i]) {
            sequence_html.append(QString("<span style=\"color:%2;\">%1</span>").arg(sequence[i]).arg(_color_best.name()));
        }
        else if (other_best_bool[i]) {
            sequence_html.append(QString("<span style=\"color:%2;\">%1</span>").arg(sequence[i]).arg(_color_other_best.name()));
        }
        else if (nobest_bool[i]) {
            sequence_html.append(QString("<span style=\"color:%2;\">%1</span>").arg(sequence[i]).arg(_color_no_best.name()));
        }
        else {
            sequence_html.append(sequence[i]);
        }
    }
    return sequence_html;
}

const std::vector<const PeptideMatch *> & PtmSampleScan::getPeptideMatchList() const {
    return _peptide_match_list;
}
